package com.ksyun;

import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.collect.ImmutableMap;
import com.ksyun.constant.Consts;
import com.ksyun.model.SmsReport;
import com.ksyun.model.SmsUp;
import com.ksyun.model.Template;
import com.ksyun.model.rdto.AddSmsSignResult;
import com.ksyun.model.rdto.BaseResult;
import com.ksyun.model.rdto.CreateTemplateResult;
import com.ksyun.model.rdto.DeleteSmsSignResult;
import com.ksyun.model.rdto.MobileLocationResult;
import com.ksyun.model.rdto.ModifySmsSignResult;
import com.ksyun.model.rdto.QuerySmsSignResult;
import com.ksyun.model.rdto.SendDiffSmsResult;
import com.ksyun.model.rdto.SendSmsResult;
import com.ksyun.model.SmsInfo;
import com.ksyun.model.rdto.SendTimingSmsResult;
import com.ksyun.model.rdto.SimpleListResult;
import com.ksyun.model.SmsDetail;
import com.ksyun.model.rdto.TemplateResult;
import com.ksyun.util.HttpUtils;
import com.ksyun.util.ParamAssemble;
import com.ksyun.util.UtilAll;
import org.apache.commons.collections.CollectionUtils;
import org.joda.time.DateTime;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class SmsService {

    private String ak;

    private String sk;

    public SmsService(String ak, String sk) {
        if (UtilAll.isNullOrEmpty(ak) || UtilAll.isNullOrEmpty(sk)) {
            throw new IllegalArgumentException("ak或sk不能为空");
        }
        this.ak = ak;
        this.sk = sk;
    }

    public SendSmsResult sendSms(String mobile, String tplId, Map<String, String> tplParams, String Content, String smsType,String signName) {
        Map<String, String> params = ParamAssemble.buildSendSmsParams(mobile, tplId, tplParams, Content, smsType, signName);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_SendSms, params);
        return HttpUtils.postForm(Consts.KSMS_HOST, finalParams, SendSmsResult.class);
    }

    public SendSmsResult batchSendSms(List<String> mobiles, String tplId, Map<String, String> tplParams, String Content, String smsType,String signName) {
        if (CollectionUtils.isEmpty(mobiles) || mobiles.size() > 200) {
            throw new IllegalArgumentException("手机号不能为空，且数量不多于200个");
        }
        String allMobiles = mobiles.stream().collect(Collectors.joining(","));
        Map<String, String> params = ParamAssemble.buildSendSmsParams(allMobiles, tplId, tplParams, Content, smsType, signName);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_BatchSendSms, params);
        return HttpUtils.postForm(Consts.KSMS_HOST, finalParams, SendSmsResult.class);
    }


    public SendDiffSmsResult sendDiffSms(List<SmsInfo> smsInfos, String tplId, String smsType,String signName) {
        if (CollectionUtils.isEmpty(smsInfos) || smsInfos.size() > 100) {
            throw new IllegalArgumentException("手机号不能为空，且数量不多于200个");
        }
        Map<String, String> params = ParamAssemble.buildSendDiffSmsParams(smsInfos, tplId, smsType, signName);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_SendDiffSms, params);
        return HttpUtils.postForm(Consts.KSMS_HOST, finalParams, SendDiffSmsResult.class);
    }

    public MobileLocationResult mobileLocation(List<String> mobiles) {
        if (CollectionUtils.isEmpty(mobiles) || mobiles.size() > 200) {
            throw new IllegalArgumentException("手机号不能为空，且数量不多于200个");
        }
        String allMobiles = mobiles.stream().collect(Collectors.joining(","));
        Map<String, String> params = ImmutableMap.of("Mobile", allMobiles);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_MobileLocation, params);
        return HttpUtils.postForm(Consts.KSMS_HOST, finalParams, MobileLocationResult.class);
    }

    public List<SmsReport> pullSmsReport(int pullSize) {
        if (pullSize > 500) {
            throw new IllegalArgumentException("单次拉取不能超过500");
        }
        Map<String, String> params = ImmutableMap.of("Size", String.valueOf(pullSize));
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_PullSmsReport, params);
        SimpleListResult<SmsReport> rs = HttpUtils.get(Consts.KSMS_HOST, finalParams, new TypeReference<SimpleListResult<SmsReport>>() {
        });
        return rs.getData();
    }

    public List<SmsUp> pullSmsUp(int pullSize) {
        if (pullSize > 500) {
            throw new IllegalArgumentException("单次拉取不能超过500");
        }
        Map<String, String> params = UtilAll.buildMap("Size", String.valueOf(pullSize));
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_PullSmsUp, params);
        SimpleListResult<SmsUp> rs = HttpUtils.get(Consts.KSMS_HOST, finalParams, new TypeReference<SimpleListResult<SmsUp>>() {
        });
        return rs.getData();
    }

    public List<SmsDetail> querySmsDetail(Date sendDate, String mobile, String sid, String extId) {
        String dateString = new DateTime(sendDate).toString(Consts.DatePattern);
        Map<String, String> params = UtilAll.buildMap("SendDate", dateString, "Mobile", mobile, "Sid", sid, "ExtId", extId);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_QuerySmsDetail, params);
        SimpleListResult<SmsDetail> rs =  HttpUtils.get(Consts.KSMS_HOST, finalParams, new TypeReference<SimpleListResult<SmsDetail>>() {
        });
        return rs.getData();
    }

    public BaseResult deliveredSms(String sid, String extId, Date sendDate) {
        String dateString = new DateTime(sendDate).toString(Consts.DatePattern);
        Map<String, String> params = UtilAll.buildMap("Sid", sid, "extId", extId, "SendDate", dateString);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_DeliveredSms, params);
        return HttpUtils.get(Consts.KSMS_HOST, finalParams, BaseResult.class);
    }

    public List<Template> listTemplates(int page, int pageSize) {
        Map<String, String> params = UtilAll.buildMap("Page", String.valueOf(page), "PageSize", String.valueOf(pageSize));
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_ListTemplates, params);
        SimpleListResult<Template> rs =  HttpUtils.get(Consts.SMS_HOST, finalParams,  new TypeReference<SimpleListResult<Template>>(){});
        return rs.getTemplates();
    }

    public TemplateResult getTemplateById(int templateId) {
        Map<String, String> params = UtilAll.buildMap("TemplateId", String.valueOf(templateId));
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_GetTemplateById, params);
        return HttpUtils.get(Consts.SMS_HOST, finalParams, TemplateResult.class);
    }

    public CreateTemplateResult createTemplate(int type, String name, String content, String description) {
        Map<String, String> params = UtilAll.buildMap("Type", String.valueOf(type), "Name", name, "Content", content, "Description", description);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_CreateTemplate, params);
        return HttpUtils.get(Consts.SMS_HOST, finalParams, CreateTemplateResult.class);
    }

    public SendTimingSmsResult sendTimingSms(int smsType, List<String> mobiles, String content, String signName,Date sendTime) {
        String sendTimeString = new DateTime(sendTime).toString(Consts.DateTimePattern);
        Map<String, String> params = UtilAll.buildMap("SmsType", String.valueOf(smsType), "Mobile", UtilAll.toJson(mobiles), "Content", content, "SendTime", sendTimeString, "SignName", signName);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_SendTimingSms, params);
        return HttpUtils.get(Consts.SMS_HOST, finalParams, SendTimingSmsResult.class);
    }

    public AddSmsSignResult addSmsSign(String signName, Integer signType, String description) {
        Map<String, String> params = UtilAll.buildMap("SignName", signName, "SignType", String.valueOf(signType), "Description", description);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_AddSmsSign, params);
        return HttpUtils.get(Consts.SMS_HOST, finalParams, AddSmsSignResult.class);
    }

    public ModifySmsSignResult modifySmsSign(String signId, String signName, Integer signType, String description) {
        Map<String, String> params = UtilAll.buildMap("SignId", signId, "SignName", signName, "SignType", String.valueOf(signType), "Description", description);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_ModifySmsSign, params);
        return HttpUtils.get(Consts.SMS_HOST, finalParams, ModifySmsSignResult.class);
    }


    public QuerySmsSignResult querySmsSign(String signId) {
        Map<String, String> params = UtilAll.buildMap("SignId", signId);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_QuerySmsSign, params);
        return HttpUtils.get(Consts.SMS_HOST, finalParams, QuerySmsSignResult.class);
    }

    public DeleteSmsSignResult deleteSmsSign(String signId) {
        Map<String, String> params = UtilAll.buildMap("SignId", signId);
        Map<String, String> finalParams = UtilAll.sign(ak, sk, Consts.region, Consts.Action_DeleteSmsSign, params);
        return HttpUtils.get(Consts.SMS_HOST, finalParams, DeleteSmsSignResult.class);
    }


}
